@@ -1,4 +1,5 @@
 use num_traits::{Float, PrimInt};
+use std::cmp::min;
 use std::mem::size_of;
 use std::ops::{BitOrAssign, BitXor};
 
@@ -82,7 +83,9 @@ where
 
     /// Push up to an usize worth of bits
     fn push_bits(&mut self, bits: usize, count: usize) {
-        self.buffer.push_bits(bits, count)
+        if count > 0 {
+            self.buffer.push_bits(bits, count)
+        }
     }
 
     /// Write a boolean into the buffer
@@ -215,8 +218,9 @@ where
         let mut bits = bits.clone();
         let bit_offset = self.bit_len() % 8;
         if bit_offset > 0 {
-            let start = bits.read_int::<u8>(8 - bit_offset)?;
-            self.push_bits(start as usize, 8 - bit_offset);
+            let bit_count = min(8 - bit_offset, bits.bits_left());
+            let start = bits.read_int::<u8>(bit_count)?;
+            self.push_bits(start as usize, bit_count);
         }
 
         while bits.bits_left() > 32 {
@@ -287,35 +291,51 @@ where
     /// Reserve some bits to be written later by splitting of two parts
     ///
     /// This allows skipping a few bits to write later
-    fn reserve(&mut self, count: usize) -> (BitWriteStream<E>, BitWriteStream<E>) {
+    pub fn reserve<
+        Err,
+        F: Fn(&mut BitWriteStream<E>, &mut BitWriteStream<E>) -> Result<(), Err>,
+    >(
+        &mut self,
+        count: usize,
+        body_fn: F,
+    ) -> Result<(), Err> {
         let (head, tail) = self.buffer.reserve(count);
-        (
-            BitWriteStream { buffer: head },
-            BitWriteStream { buffer: tail },
-        )
+        let mut head = BitWriteStream { buffer: head };
+        let mut tail = BitWriteStream { buffer: tail };
+        body_fn(&mut head, &mut tail)
     }
 
     /// Write the length of a section before the section
-    pub fn reserve_length<F: Fn(&mut BitWriteStream<E>) -> Result<()>>(
+    pub fn reserve_length<Err: From<BitError>, F: Fn(&mut BitWriteStream<E>) -> Result<(), Err>>(
         &mut self,
         length_bit_size: usize,
         body_fn: F,
-    ) -> Result<()> {
-        let (mut head, mut tail) = self.reserve(length_bit_size);
+    ) -> Result<(), Err> {
+        let (head, tail) = self.buffer.reserve(length_bit_size);
+        let mut head = BitWriteStream { buffer: head };
+        let mut tail = BitWriteStream { buffer: tail };
+
         let start = tail.bit_len();
         body_fn(&mut tail)?;
         let end = tail.bit_len();
         let bit_len = end - start;
-        head.write_sized(&bit_len, length_bit_size)
+        head.write_sized(&bit_len, length_bit_size)?;
+        Ok(())
     }
 
     /// Write the length in bytes of a section before the section, the section will be 0 padded to an even byte length
-    pub fn reserve_byte_length<F: Fn(&mut BitWriteStream<E>) -> Result<()>>(
+    pub fn reserve_byte_length<
+        Err: From<BitError>,
+        F: Fn(&mut BitWriteStream<E>) -> Result<(), Err>,
+    >(
         &mut self,
         length_bit_size: usize,
         body_fn: F,
-    ) -> Result<()> {
-        let (mut head, mut tail) = self.reserve(length_bit_size);
+    ) -> Result<(), Err> {
+        let (head, tail) = self.buffer.reserve(length_bit_size);
+        let mut head = BitWriteStream { buffer: head };
+        let mut tail = BitWriteStream { buffer: tail };
+
         let start = tail.bit_len();
         body_fn(&mut tail)?;
         let end = tail.bit_len();
@@ -326,6 +346,7 @@ where
 
         let byte_len = (bit_len + pad_len) / 8;
 
-        head.write_sized(&byte_len, length_bit_size)
+        head.write_sized(&byte_len, length_bit_size)?;
+        Ok(())
     }
 }
